package logic

import (
	"encoding/json"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/product/internal/svc"
	"fusenapi/server/product/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetProductDesignLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetProductDesignLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetProductDesignLogic {
	return &GetProductDesignLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetProductDesignLogic) GetProductDesign(req *types.GetProductDesignReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if userinfo.GetIdType() != auth.IDTYPE_User {
		return resp.SetStatusWithMessage(basic.CodeUnAuth, "please login first")
	}
	if req.Sn == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "param sn is required")
	}
	//获取设计数据
	productDesignModel := gmodel.NewFsProductDesignModel(l.svcCtx.MysqlConn)
	designInfo, err := productDesignModel.FindOneBySn(l.ctx, req.Sn, userinfo.UserId)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get design info")
	}
	if designInfo.Id == 0 {
		return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "design info is not exists")
	}
	//获取产品尺寸信息
	productSizeModel := gmodel.NewFsProductSizeModel(l.svcCtx.MysqlConn)
	sizeInfo, err := productSizeModel.FindOne(l.ctx, *designInfo.SizeId)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get product size info")
	}
	if sizeInfo.Id == 0 {
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "product size info is not exists")
	}
	//获取模板信息
	productTemplateV2Model := gmodel.NewFsProductTemplateV2Model(l.svcCtx.MysqlConn)
	productTemplateV2Info, err := productTemplateV2Model.FindOne(l.ctx, *designInfo.TemplateId)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get template info")
	}
	if productTemplateV2Info.Id == 0 {
		return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "template info is not exists")
	}
	//解析json
	var info types.ProductDesignInfo
	if err = json.Unmarshal([]byte(*designInfo.Info), &info); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to parse design info")
	}
	return resp.SetStatus(basic.CodeOK)
}
